/*
** Module   : HandleResource.java
** Abstract : Implementation of common attributes and APIs for all resources. 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CA  20130221 Created initial version.
** 002 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
*/

package com.goldencode.p2j.util;

/**
 * This acts as a superclass which implements the of the CommonHandle interface:
 * <table>
 * <tr>
 *    <th>attribute or base method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>TYPE</td>
 *    <td>{@link #getResourceType()}</td>
 * </tr>
 * <tr>
 *    <td>Unknown status</td>
 *    <td>{@link #unknown()}</td>
 * </tr>
 * <tr>
 *    <td>Read only field error</td>
 *    <td>{@link #readOnlyError(String)}</td>
 * </tr>
 * <tr>
 *    <td>Invalid attribute access</td>
 *    <td>{@link #invalidAttribute(String)}</td>
 * </tr>
 * </tr>
 * </table>
 * <p>
 * The abstract class also provides setters for the implemented attributes 
 * which are not defined inside the interfaces to be used for initializing of
 * those attributes
 */
public abstract class HandleResource
implements CommonHandle
{
   /** Variable for holding the type attribute */
   private String type = null;
   
   /** Variable for holding unknown state */
   private boolean isUnknown = true;
   
   /**
    * Get the type of this handle.
    * 
    * @return   See above.
    */
   public character getResourceType()
   {
      return new character(type);
   }
   
   /**
    * Sets the type of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param    type
    *           The handle type to be set.
    */
   public void setResourceType(String type)
   {
      if (this.type == null)
      {
         this.type = type;
      }
   }

   /**
    * Reports if this object is unknown.  
    *
    * @return   <code>true</code> if object is unknown.
    */
   public boolean unknown()
   {
      return isUnknown;
   }
   
   /**
    * Sets whether this object is unknown.This method is outside interfaces and 
    * only meant for initializing this within the implementation.
    * 
    * @param isUnknown
    *        The handle type to be set.
    */
   public void setUnknown(boolean isUnknown)
   {
      this.isUnknown = isUnknown;
   }

   /**
    * To be used only in cases when a setter for a read-only attribute or a not-supported  
    * attribute is forced to be implemented by the resource. It displays appropriate error 
    * message.
    * 
    * @param    attr
    *           The read-only attribute.
    */
   public void readOnlyError(String attr)
   {
      handle.readOnlyError(new handle(this), attr);
   }

   /**
    * To be used only in cases a getter for a not-supported attribute is forced to be implemented
    * by the resource. It displays appropriate error message.
    * 
    * @param    attr
    *           The not-supported attribute attribute.
    */
   protected void invalidAttribute(String attr)
   {
      handle.invalidAttribute(attr, new handle(this));
   }
}
